/*
(C) 2002 Advanced Micro Devices, Inc. 
** YOUR USE OF THIS LIBRARY IS SUBJECT TO THE TERMS 
    AND CONDITIONS OF THE GNU LESSER GENERAL PUBLIC 
    LICENSE FOUND IN THE "README" FILE THAT IS INCLUDED WITH 
    THIS LIBRARY**
*/

#include "libm_amd.h"
#include "libm_util_amd.h"

#define USE_VAL_WITH_FLAGS
#define USE_SCALEFLOAT_1
#define USE_SCALEUPFLOAT128
#include "libm_inlines_amd.h"
#undef USE_SCALEUPFLOAT128
#undef USE_SCALEFLOAT_1
#undef USE_VAL_WITH_FLAGS

float __atanf(float fy)
{

  /* Some constants and split constants. */

  static double piby2 = 1.5707963267948966e+00; /* 0x3ff921fb54442d18 */

  double c, v, s, q, z;
  unsigned int ynan;

  double y = fy;

  /* Find properties of argument y. */

  unsigned long uy, auy, yneg;

  GET_BITS_DP64(y, uy);
  auy = uy & ~SIGNBIT_DP64;
  yneg = uy & SIGNBIT_DP64;

  v = y;
  if (yneg) v = -y;

  /* Argument reduction to range [-7/16,7/16] */

  if (auy < 0x3fdc000000000000) /* v < 7./16. */
    {
      y = v;
      c = 0.0;
    }
  else if (auy < 0x3fe6000000000000) /* v < 11./16. */
    {
      y = (2*v-1.0)/(2.0+v);
      /* c = arctan(0.5) */
      c = 4.63647609000806093515e-01; /* 0x3fddac670561bb4f */
    }
  else if (auy < 0x3ff3000000000000) /* v < 19./16. */
    {
      y = (v-1.)/(1.0+v);
      /* c = arctan(1.) */
      c = 7.85398163397448278999e-01; /* 0x3fe921fb54442d18 */
    }
  else if (auy < 0x4003800000000000) /* v < 39./16. */
    {
      y = (v-1.5)/(1.0+1.5*v);
      /* c = arctan(1.5) */
      c = 9.82793723247329054082e-01; /* 0x3fef730bd281f69b */
    }
  else
    {

      ynan = (auy > PINFBITPATT_DP64);  

      if (ynan) return y + y;
      else if (v > 0x4c80000000000000)
	{ /* abs(y) > 2^26 => arctan(1/y) is 
	     insignificant compared to piby2 */
	  if (yneg) return val_with_flags(-piby2, AMD_F_INEXACT);
	  else return val_with_flags(piby2, AMD_F_INEXACT);
	}

      y = -1.0/v;
      /* c = arctan(infinity) */
      c = 1.57079632679489655800e+00; /* 0x3ff921fb54442d18 */
    }

  /* Core approximation: Remez(2,2) on [-7/16,7/16] */

  s = y*y;
  q = y*s*
    (0.296528598819239217902158651186e0 + 
     (0.192324546402108583211697690500e0 + 
       0.470677934286149214138357545549e-2*s)*s)/
    (0.889585796862432286486651434570e0 +
     (0.111072499995399550138837673349e1 + 
       0.299309699959659728404442796915e0*s)*s);

  z = c - (q - y);

  if (yneg) z = -z;
  return z;
}

weak_alias (__atanf, atanf)
